#!/usr/bin/env newlisp
;;
;; RSS 2.0 feed for newlisp-wiki, RSS 0.92 discontinued for newLISP Wiki
;;
;; place this file in the main directory of your newLISP Wiki/CMS as: rss.cgi
;;
;; v. 1.1 RSS 0.92
;; v. 2.0 upgraded to RSS 2.0 May 20th, 2006
;; v. 2.1 changes for newLISP v.10.0
;;
;; For this to work optimal, setup should be done first in newlisp-wiki
;; to fill out the variables: rss-title, rss-desrciption, rss-link
;; and rss-copyright. If these fields are not found rss.cgi takes
;; rss-title and rss-description from the page 'Browser_Title'
;; and rss-link is compoes from the current accessed URL of this script,
;; the rss-copyright is set to the domain name of the script URL
;;
;; If no parameter page is given when requesting rss.cgi then rss.cgi?News
;; is assumed and the page 'News' will be processed
;;

;; prepare output
;;
(print "Content-type: text/xml\n\n")

(load "pages/setup.lsp")

;; if no page for rss is specified assume News
(set 'rss-page-name (env "QUERY_STRING"))
(if (or (not rss-page-name) (= rss-page-name "")) (set 'rss-page-name "News"))

;; if no rss header info is supplied, make a guess
(if (not SETUP:rss-title) (set 'SETUP:rss-title (read-file "pages/Browser_Title")))
(replace {\[.*\]} SETUP:rss-title "" 512) ;; remove wiki tags
(if (not SETUP:rss-description) (set 'SETUP:rss-description SETUP:rss-title))
(set 'url (env "SCRIPT_URI"))
(if (not url) (set 'url ""))
(if (not SETUP:rss-link) (set 'SETUP:rss-link (join (chop (parse url "/")) "/")))
(if (not SETUP:rss-copyright) 
	(set 'SETUP:rss-copyright (append "copyright (c) " (nth 2 (parse url "/")) 
			", 2004. All rights reserved.")))

;; write RSS header and channel info
;;
;(set 'lastBD (parse (date) " +" 0))
(set 'lastBD (date (date-value) 300 "%a, %d %b %Y %H:%M:%S UCT") )
(println {<?xml version="1.0" encoding="UTF-8"?>})
(println {<!-- generator="newLISP Wiki/CMS rss.cgi v. 1.0" -->})
(println {<rss version="2.0">})
(println {    <channel>})
(println {        <title>} SETUP:rss-title {</title>})
(println {        <description>} SETUP:rss-description {</description>})
(println {        <link>} SETUP:rss-link {</link>})
(println {        <lastBuildDate>} lastBD {</lastBuildDate>})
(println {        <managingEditor>lutz@nuevatec.com (Lutz Mueller)</managingEditor>})

;; get a list of all posts
;;
(set 'page (read-file (append "pages/" rss-page-name)))
(if (not page) (set 'page ""))

(set 'start (find {\[post:.*\]} page 512))
(if (not start) (begin
	(println {    </channel>})
	(println {</rss>})
        (exit)))

(inc start (length $0))
(set 'posts (chop (parse (slice page start) {<\!--.*\..*\..*\..*-->} 512)))

;; iterate through posts
(dolist (post posts)
    (if (find "=====(.*)=====" post 512) (set 'subject (trim $1)) (set 'subject ""))
    (if (find "find=(.*) #" post 512) (set 'perma-link $1) (set 'perma-link ""))
    (if (find "======(.*)\\[http:" post 512) 
        (begin
            (set 'pubD (3 (parse $1 " +" 0))) 
            (set 'pubD (format "%s, %s %s %s %s UCT" (pubD 0) (pubD 2) (pubD 1) (pubD 4) (pubD 3)))) )
    (set 'description "")
    (println {    <item>})
    (println {        <title>} subject {</title>})
    (println {        <link>} SETUP:rss-link {/index.cgi?find=} perma-link {</link>})
    (println {        <pubDate>} pubD {</pubDate>})
    (println {        <description>} description {</description>})
    (println {    </item>}))

(println {    </channel>})
(println {</rss>})

(exit)

;; eof ;;

